@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from pathlib import Path
 from typing import Any, Dict, List, Literal, Optional, Union
 
@@ -38,6 +40,40 @@ class MultiArchitectureRemoteFile(Base):
     )
 
 
+class SourceDestinationStr(str):
+    source: str
+    destination: str
+
+    @classmethod
+    def __get_validators__(cls) -> Any:
+        yield cls.validate
+
+    @classmethod
+    def validate(cls, v: str) -> "SourceDestinationStr":
+        if not isinstance(v, str):
+            raise TypeError("string required")
+        if len(paths := v.split(":")) != 2:
+            raise TypeError("string must have one ':' separator")
+        string = cls(v)
+        string.source = paths[0]
+        string.destination = paths[1]
+        return string
+
+    def __repr__(self) -> str:
+        return (
+            f"SourceDestinationStr(source={self.source}, destination={self.destination})"
+        )
+
+
+class HereDocument(Base):
+    content: str = Field(..., description="File content")
+    path: str = Field(..., description="Path where the file will be written")
+
+    @property
+    def destination(self) -> str:
+        return self.path
+
+
 class Blueprint(Base):
     name: str = Field(..., description="Package name")
     version: str = Field(..., description="Package name")
@@ -60,6 +96,7 @@ class Blueprint(Base):
         description="Describe a file (or a file per architecture) to download before "
         "running the build script",
     )
+    install: List[Union[HereDocument, SourceDestinationStr]] = Field(default_factory=list)
     script: List[str] = Field(default_factory=list, description="Build instructions")
 
     @validator("*", pre=True)
@@ -86,14 +123,14 @@ class Blueprint(Base):
         else:
             return [self.arch]
 
-    def render_string(self, string: str, **kwargs: Optional[str]) -> str:
+    def render_string(self, string: str, **kwargs: Optional[str | Path]) -> str:
         version = kwargs.pop("version", None)
         version = version or self.version
         return environment.from_string(string).render(
             name=self.name,
             arch=self.arch,
             version=version,
-            **kwargs,
+            **(self._get_additional_variables() | kwargs),
         )
 
     def render_fetch(self, version: Optional[str] = None) -> Optional[RemoteFile]:
@@ -109,15 +146,11 @@ class Blueprint(Base):
             self.fetch.url,
             version=version,
             sha256=sha256,
-            **self._get_additional_variables(),
         )
         return RemoteFile(url=url, sha256=sha256)
 
-    def render_script(self, src: Path = None) -> List[str]:
-        return [
-            self.render_string(line, src=str(src), **self._get_additional_variables())
-            for line in self.script
-        ]
+    def render_script(self, src: Path, debian: Path) -> List[str]:
+        return [self.render_string(line, src=src, debian=debian) for line in self.script]
 
 
 class Configuration(Base):
